@@ -1,6 +1,11 @@
 //! CLI option parsing.
 
-use std::{env, ffi::OsStr, path::Path, path::PathBuf};
+use std::{
+    env,
+    ffi::OsStr,
+    fmt::Display,
+    path::{Path, PathBuf},
+};
 
 use clap::{crate_authors, crate_description, Arg, Command};
 use const_format::formatcp;
@@ -312,6 +317,73 @@ pub(crate) enum FileArgument {
     DevNull,
 }
 
+impl FileArgument {
+    pub(crate) fn permissions(&self) -> Option<FilePermissions> {
+        match self {
+            FileArgument::NamedPath(path) => {
+                // When used with `git difftool`, the first argument
+                // is a temporary file that always has the same
+                // permissions. That doesn't mean the file permissions
+                // have changed, so we shouldn't compare.
+                if is_git_tmpfile(path) {
+                    return None;
+                }
+
+                let metadata = std::fs::metadata(path).ok()?;
+                Some(metadata.permissions().into())
+            }
+            FileArgument::Stdin => None,
+            FileArgument::DevNull => None,
+        }
+    }
+}
+
+/// A cross-platform representation of file permissions.
+///
+/// We're only interested in whether two permissions are the same, and
+/// how to display them, so internally this is just a human-friendly
+/// string.
+#[derive(Debug, Eq, PartialEq)]
+pub(crate) struct FilePermissions(String);
+
+impl Display for FilePermissions {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        write!(f, "{}", self.0)
+    }
+}
+
+impl TryFrom<&OsStr> for FilePermissions {
+    type Error = ();
+
+    fn try_from(s: &OsStr) -> Result<Self, Self::Error> {
+        if s == "." {
+            Err(())
+        } else {
+            Ok(Self(s.to_string_lossy().into_owned()))
+        }
+    }
+}
+
+#[cfg(unix)]
+impl From<std::fs::Permissions> for FilePermissions {
+    fn from(perms: std::fs::Permissions) -> Self {
+        use std::os::unix::fs::PermissionsExt;
+        Self(format!("{:o}", perms.mode()))
+    }
+}
+
+#[cfg(not(unix))]
+impl From<std::fs::Permissions> for FilePermissions {
+    fn from(perms: std::fs::Permissions) -> Self {
+        let s = if perms.readonly() {
+            "readonly"
+        } else {
+            "read-write"
+        };
+        Self(s.to_owned())
+    }
+}
+
 fn try_canonicalize(path: &Path) -> PathBuf {
     path.canonicalize().unwrap_or_else(|_| path.into())
 }
@@ -352,12 +424,16 @@ impl FileArgument {
             FileArgument::NamedPath(PathBuf::from(arg))
         }
     }
+}
 
-    pub(crate) fn display(&self) -> String {
+impl Display for FileArgument {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            FileArgument::NamedPath(path) => relative_to_current(path).display().to_string(),
-            FileArgument::Stdin => "(stdin)".to_string(),
-            FileArgument::DevNull => "/dev/null".to_string(),
+            FileArgument::NamedPath(path) => {
+                write!(f, "{}", relative_to_current(path).display())
+            }
+            FileArgument::Stdin => write!(f, "(stdin)"),
+            FileArgument::DevNull => write!(f, "/dev/null"),
         }
     }
 }
@@ -374,6 +450,8 @@ pub(crate) enum Mode {
         /// The path where we can read the RHS file. This is often a
         /// temporary file generated by source control.
         rhs_path: FileArgument,
+        lhs_permissions: Option<FilePermissions>,
+        rhs_permissions: Option<FilePermissions>,
         /// The path that we show to the user.
         display_path: String,
         /// If this file has been renamed, a description of the change.
@@ -432,9 +510,30 @@ fn common_path_suffix(lhs_path: &Path, rhs_path: &Path) -> Option<String> {
     }
 }
 
+/// Does `path` look like "/tmp/git-blob-abcdef/modified_field.txt"?
+fn is_git_tmpfile(path: &Path) -> bool {
+    let Ok(rel_path) = path.strip_prefix(std::env::temp_dir()) else {
+        return false;
+    };
+
+    let components: Vec<_> = rel_path.components().collect();
+    if components.len() != 2 {
+        return false;
+    }
+
+    components[0]
+        .as_os_str()
+        .to_string_lossy()
+        .starts_with("git-blob-")
+}
+
 fn build_display_path(lhs_path: &FileArgument, rhs_path: &FileArgument) -> String {
     match (lhs_path, rhs_path) {
         (FileArgument::NamedPath(lhs), FileArgument::NamedPath(rhs)) => {
+            if is_git_tmpfile(lhs) {
+                return rhs.display().to_string();
+            }
+
             match common_path_suffix(lhs, rhs) {
                 Some(common_suffix) => common_suffix,
                 None => rhs.display().to_string(),
@@ -635,23 +734,38 @@ pub(crate) fn parse_args() -> Mode {
     info!("CLI arguments: {:?}", args);
 
     // TODO: document these different ways of calling difftastic.
-    let (display_path, lhs_path, rhs_path, renamed) = match &args[..] {
+    let (display_path, lhs_path, rhs_path, lhs_permissions, rhs_permissions, renamed) = match &args
+        [..]
+    {
         [lhs_path, rhs_path] => {
             let lhs_arg = FileArgument::from_cli_argument(lhs_path);
             let rhs_arg = FileArgument::from_cli_argument(rhs_path);
             let display_path = build_display_path(&lhs_arg, &rhs_arg);
-            (display_path, lhs_arg, rhs_arg, None)
+
+            let lhs_permissions = lhs_arg.permissions();
+            let rhs_permissions = rhs_arg.permissions();
+
+            (
+                display_path,
+                lhs_arg,
+                rhs_arg,
+                lhs_permissions,
+                rhs_permissions,
+                None,
+            )
         }
-        [display_path, lhs_tmp_file, _lhs_hash, _lhs_mode, rhs_tmp_file, _rhs_hash, _rhs_mode] => {
+        [display_path, lhs_tmp_file, _lhs_hash, lhs_mode, rhs_tmp_file, _rhs_hash, rhs_mode] => {
             // https://git-scm.com/docs/git#Documentation/git.txt-codeGITEXTERNALDIFFcode
             (
                 display_path.to_string_lossy().to_string(),
                 FileArgument::from_path_argument(lhs_tmp_file),
                 FileArgument::from_path_argument(rhs_tmp_file),
+                FilePermissions::try_from(*lhs_mode).ok(),
+                FilePermissions::try_from(*rhs_mode).ok(),
                 None,
             )
         }
-        [old_name, lhs_tmp_file, _lhs_hash, _lhs_mode, rhs_tmp_file, _rhs_hash, _rhs_mode, new_name, _similarity] =>
+        [old_name, lhs_tmp_file, _lhs_hash, lhs_mode, rhs_tmp_file, _rhs_hash, rhs_mode, new_name, _similarity] =>
         {
             // Rename file.
             // TODO: where does git document these 9 arguments?
@@ -664,6 +778,8 @@ pub(crate) fn parse_args() -> Mode {
                 new_name,
                 FileArgument::from_path_argument(lhs_tmp_file),
                 FileArgument::from_path_argument(rhs_tmp_file),
+                FilePermissions::try_from(*lhs_mode).ok(),
+                FilePermissions::try_from(*rhs_mode).ok(),
                 Some(renamed),
             )
         }
@@ -724,6 +840,8 @@ pub(crate) fn parse_args() -> Mode {
         language_overrides,
         lhs_path,
         rhs_path,
+        lhs_permissions,
+        rhs_permissions,
         display_path,
         renamed,
     }
