@@ -66,7 +66,9 @@ pub(crate) enum Language {
     Ruby,
     Rust,
     Scala,
+    Scheme,
     Scss,
+    Smali,
     Solidity,
     Sql,
     Swift,
@@ -155,6 +157,8 @@ pub(crate) fn language_name(language: Language) -> &'static str {
         Ruby => "Ruby",
         Rust => "Rust",
         Scala => "Scala",
+        Scheme => "Scheme",
+        Smali => "Smali",
         Scss => "SCSS",
         Solidity => "Solidity",
         Sql => "SQL",
@@ -312,7 +316,7 @@ pub(crate) fn language_globs(language: Language) -> Vec<glob::Pattern> {
             "Makefile.am",
             "Makefile.boot",
             "Makefile.frag",
-            "Makefile.in",
+            "Makefile*.in",
             "Makefile.inc",
             "Makefile.wat",
             "makefile",
@@ -341,6 +345,8 @@ pub(crate) fn language_globs(language: Language) -> Vec<glob::Pattern> {
         ],
         Rust => &["*.rs"],
         Scala => &["*.scala", "*.sbt", "*.sc"],
+        Scheme => &["*.scm", "*.sch", "*.ss"],
+        Smali => &["*.smali"],
         Scss => &["*.scss"],
         Solidity => &["*.sol"],
         Sql => &["*.sql", "*.pgsql"],
@@ -541,7 +547,7 @@ fn from_emacs_mode_header(src: &str) -> Option<Language> {
 /// Try to guess the language based on a shebang present in the source.
 fn from_shebang(src: &str) -> Option<Language> {
     lazy_static! {
-        static ref RE: Regex = Regex::new(r"#!(?:/usr/bin/env )?([^ ]+)").unwrap();
+        static ref RE: Regex = Regex::new(r"#! *(?:/usr/bin/env )?([^ ]+)").unwrap();
     }
     if let Some(first_line) = src.lines().next() {
         if let Some(cap) = RE.captures(first_line) {
@@ -629,6 +635,12 @@ mod tests {
         assert_eq!(guess(path, "#!/usr/bin/env python", &[]), Some(Python));
     }
 
+    #[test]
+    fn test_guess_by_shebang_with_space() {
+        let path = Path::new("foo");
+        assert_eq!(guess(path, "#! /bin/sh", &[]), Some(Bash));
+    }
+
     #[test]
     fn test_guess_by_emacs_mode() {
         let path = Path::new("foo");
